/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package test;

import java.io.IOException;
import org.infinispan.Cache;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.transaction.tm.DummyTransactionManager;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.BeforeClass;

/**
 *
 */
public class TestReplace {

    @BeforeClass
    public static void init() {
        System.setProperty("java.net.preferIPv4Stack", "true");
    }
    Cache cache;
    Cache cacheRepl;
    DefaultCacheManager cacheManager;
    DefaultCacheManager cacheManagerRepl;
    DummyTransactionManager transactionManager = new DummyTransactionManager();

    @Before
    public void createCache() throws InterruptedException, IOException {
        cacheManager = new org.infinispan.manager.DefaultCacheManager("test-cluster_1.xml");
        System.out.println("===Waiting for clustering join");
        cache = cacheManager.getCache("TEST_CACHE");

        cacheManagerRepl = new org.infinispan.manager.DefaultCacheManager("test-cluster_1.xml");
        cacheRepl = cacheManagerRepl.getCache("TEST_CACHE");
        cacheManager.start();
        cacheManagerRepl.start();
        Thread.sleep(1000);
    }

    @After
    public void shutdown() {
        cacheManager.stop();
        cacheManagerRepl.stop();
    }

    @Test
    public void testReplaceWithOldValue() throws Exception {
        System.out.println("===Launching Test");
        cache.put(new StringKey("key1"), 1);
        for (int i = 0; i < 3; i++) {
            increase();
        }
        Thread.sleep(1000);
        Assert.assertEquals(4, cacheRepl.get(new StringKey("key1")));
    }

    @Test
    public void testMultipleThreads() throws InterruptedException {
        StringKey key1 = new StringKey("key1");
        cache.put(key1, 1);
        int count = 2000;
        XThread[] threads = new XThread[50];
        for (int i = 0; i < threads.length; i++) {
            threads[i] = new XThread(count);
            threads[i].start();
        }

        for (int i = 0; i < threads.length; i++) {
            threads[i].join();
        }
        Thread.sleep(1000);
        Assert.assertEquals(1 + count * threads.length, cacheRepl.get(key1));
    }

    class XThread extends Thread {

        private int count;

        public XThread(int count) {
            this.count = count;
        }

        @Override
        public void run() {
            for (int i = 0; i < count; i++) {
                try {
                    increase();
                } catch (Exception e) {
                    e.printStackTrace();
                    break;
                }
            }
        }
    }

    private void increase() throws Exception {
        transactionManager.begin();
        try {
            StringKey key = new StringKey("key1");
            cache.getAdvancedCache().lock(key);
            Integer value = (Integer) cache.get(key);
            cache.put(key, value + 1);
            transactionManager.commit();
        } catch (Exception e) {
            transactionManager.rollback();
            throw e;
        }

    }
}
